#ifndef I_WMGET_H
#define I_WMGET_H
/*
    wmget - A background download manager as a Window Maker dock app
    Copyright (c) 2001-2002 Aaron Trickey <aaron@amtrickey.net>

    Permission is hereby granted, free of charge, to any person
    obtaining a copy of this software and associated documentation files
    (the "Software"), to deal in the Software without restriction,
    including without limitation the rights to use, copy, modify, merge,
    publish, distribute, sublicense, and/or sell copies of the Software,
    and to permit persons to whom the Software is furnished to do so,
    subject to the following conditions:

    The above copyright notice and this permission notice shall be
    included in all copies or substantial portions of the Software.

    THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND,
    EXPRESS OR IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF
    MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE AND
    NONINFRINGEMENT.  IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY
    CLAIM, DAMAGES OR OTHER LIABILITY, WHETHER IN AN ACTION OF CONTRACT,
    TORT OR OTHERWISE, ARISING FROM, OUT OF OR IN CONNECTION WITH THE
    SOFTWARE OR THE USE OR OTHER DEALINGS IN THE SOFTWARE.

    ********************************************************************
    wmget.h - Common definitions for all wmget modules

    This file defines the public entry points in server.c, request.c,
    retrieve.c, and wmget.c, as well as some common constants, global
    variables, and the structure of the shared memory segment.
*/


#include <sys/param.h>
#include <curl/curl.h>      /* purely for the version string! */

/* Important: the Makefile greps the source to extract WMGET_VERSION.
 * So don't change its format or anything without checking there first.
 */
#define WMGET_VERSION           "0.4.4"
#define WMGET_VERSION_BANNER    "wmget " WMGET_VERSION \
                                ", compiled with libcurl " \
                                        LIBCURL_VERSION

#define MAXURL          1024
#define MAXCMDLEN       (MAXURL + 10)
#define MAXCMDARGS      10

#define MAX_ACTIVE_JOBS 4       /* a number constrained by the UI */
#define MAX_QUEUED_JOBS 20

/* Command language */

/* The GET command and its arguments: */
#define CMD_GET                 "GET"
#define ARG_GET_URL              "FROM"
#define ARG_GET_DISPLAY          "DISPLAY"
#define ARG_GET_TARGET           "TO"
#define ARG_GET_START_FROM       "START"
#define ARG_GET_OVERWRITE_OK     "OVERWRITE"

#define CMD_CANCEL              "CANCEL"
#define ARG_CANCEL_JOBID         "JOBID"

#define CMD_LIST                "LIST"

#define RESPONSE_JOB_ACCEPTED   "ACCEPTED"
#define RESPONSE_JOB_CANCELED   "CANCELED"
#define RESPONSE_LIST_COMING    "JOBLIST"
#define RESPONSE_ERROR          "ERROR"

typedef enum {
    J_EMPTY,        /* this job slot is empty */
    J_INIT,         /* slot has been taken, job not yet started */
    J_RUNNING,      /* this job is running */
    J_PAUSED,       /* this job is paused */
    J_STOPPING,     /* a stop request has come from the user */
    J_COMPLETE,     /* job complete, cleaning up */
} JobStatus;

typedef unsigned long job_id_t;

typedef struct {
    job_id_t job_id;
    JobStatus status;

    unsigned long progress;
    unsigned long prog_max;
    int stop_request;

    char display_name[10];
    char srcurl[MAXURL + 1];
    char target[MAXPATHLEN + 1];
    size_t continue_download_from;
    int opt_overwrite;
} Job;

typedef struct {
    Job jobs[MAX_ACTIVE_JOBS];
} Shmem;

/* iq.c */
extern int iq_server_init (void);       /* called once, by server  */
extern FILE *iq_server_accept (void);   /* returns new cxn or NULL */
extern FILE *iq_client_connect (void);  /* called by each client   */
extern int iq_get_listen_fd (void);     /* so you can select/poll  */

/* server.c */
extern Shmem *shmem;

extern int server (int argc, char *argv[]);

/* request.c */
extern int request (int argc, char *const argv[]);

/* cancel.c */
extern int cancel (int argc, char *const argv[]);

/* list.c */
extern int list (int argc, char *const argv[]);

/* retrieve.c */
extern int retrieve (Job *job);

/* wmget.c */
extern const char *home_directory (void);
extern int build_and_check_target (Job *j);

enum OutputLevel {
    OL_SILENT,
    OL_NORMAL,
    OL_DEBUG,
};

extern enum OutputLevel output_level;

extern void debug_dump_job (Job *job);

extern void error (const char *fmt, ...);
extern void error_sys (const char *fmt, ...);
extern void info (const char *fmt, ...);
extern void debug (const char *fmt, ...);


#endif /* I_WMGET_H */
